// TO DO: check integrity of the image before executing it


#include <stdio.h>
//#include <string.h>
#include "riscv.h"

#define IRAM_SIZE       65536
#define LOADER_SIZE     512
#define APP_IMAGE_TEXT	0x60000000
#define APP_IMAGE_DATA	0x60020000
#define LOADER_IMAGE    0x60012200

#define print_line() print_char('\n');print_char('\r')

typedef struct uart_reg  {
  uint32_t flags;
  uint32_t buffer;
} uart_reg;

void  __attribute__((__section__(".launcher_call"))) print_string (char * str, unsigned char len) {
 
	for (char i=0; i < len; i++) 
		print_char(str[i]);
        
	
}

void   __attribute__((always_inline)) print_char (char c)  {
	volatile uart_reg * uart_tx = (uart_reg *) 0x82000000;
	while (!uart_tx->flags);
	uart_tx->buffer = c;
	uart_tx->flags = 2;

}


void  __attribute__((__section__(".launcher_call"))) print_word (uint32_t  word) {
	int8_t i;
	uint8_t disp;
	uint32_t temp;
	char c;
	
	for (i=7; i >= 0; i--) {

	  disp = ((word >> (i<<2)) & 0xf);
		if (disp <= 9 )
			c = '0'+ disp;
		else
			c = 'A' + disp - 10; 
 
		print_char (c);
	}

}


uint32_t  __attribute__((__section__(".launcher_call"))) envm_read (uint32_t * addr) {

	volatile uint32_t * wb_envm_dev = 0x87000000;

	while (wb_envm_dev[0] & 0x1);
	wb_envm_dev[1] = addr;
	while (*wb_envm_dev & 0x1);

	return wb_envm_dev[2];
}


void  __attribute__((__section__(".launcher"))) iram_write (uint32_t * addr, uint32_t data) {

	volatile uint32_t * wb_iram_dev = 0x87000010;

	
	wb_iram_dev[0] = addr;
	wb_iram_dev[1] = data;
	asm ("nop");
	//	asm("nop");
	//	asm("nop");
	//  asm("nop");

}




void load_image ();


void main(void)
{

	
   const char boot_logo [5] = {'b','o','o','t'};
	

	volatile char  * p_wd = 0x81000000;
	void (*func)(void) = IRAM_SIZE - LOADER_SIZE + (&load_image - 0x200) ;
	uint32_t i, data = 0;


	//print_line();
	print_string (boot_logo, 4);
	print_line();



	for (i=1024; i < (1<<16); i=i+4)
		iram_write(i, 0);

	for (i=0; i < 1024; i=i+4) {
		data = envm_read (LOADER_IMAGE + i);
		iram_write(IRAM_SIZE-LOADER_SIZE+i, data);
	}

	*p_wd = 1;

	(*func)();	// jump to (relocated) load_image()



	while(1);
//	*p_wd = 1; //watchdog



}


void __attribute__((__section__(".launcher")))  load_image () {
	const char done [5] = {'D','o','n', 'e'};
	uint32_t i, data = 0;
	uint32_t * dram = 0;
	void (*func)(void) = 0 ;

	for (i=0; i < IRAM_SIZE-LOADER_SIZE; i=i+4) {
		data = envm_read (APP_IMAGE_TEXT + i);
		iram_write (i, data);
	}

	for (i=0; i < (1<<16)-1024; i=i+4) {
		data = envm_read (APP_IMAGE_DATA + i);
		*dram++ = data;
	}
	print_string(done, 4);
	

	(*func)(); // run app

	while(1);
	
	
}

